@@ -1,10 +1,10 @@
 use strictures 2;
-package OpenAPI::Modern; # git description: v0.050-8-g0e8d7b2
+package OpenAPI::Modern; # git description: v0.051-6-g33162f8
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate HTTP requests and responses against an OpenAPI v3.1 document
 # KEYWORDS: validation evaluation JSON Schema OpenAPI v3.1 Swagger HTTP request response
 
-our $VERSION = '0.051';
+our $VERSION = '0.052';
 
 use 5.020;
 use utf8;
@@ -466,15 +466,11 @@ sub find_path ($self, $options) {
   return 1;
 }
 
-sub recursive_get ($self, $uri_reference) {
+sub recursive_get ($self, $uri_reference, $entity_type = undef) {
   my $base = $self->openapi_uri;
   my $ref = $uri_reference;
   my ($depth, $schema);
 
-  my $entity_type;
-  $entity_type = $self->openapi_document->get_entity_at_location(Mojo::URL->new($uri_reference)->fragment)
-    if $uri_reference =~ /^#/;
-
   while ($ref) {
     die 'maximum evaluation depth exceeded' if $depth++ > $self->evaluator->max_traversal_depth;
     my $uri = Mojo::URL->new($ref)->to_abs($base);
@@ -482,8 +478,8 @@ sub recursive_get ($self, $uri_reference) {
     my $schema_info = $self->evaluator->_fetch_from_uri($uri);
 
     die('unable to find resource ', $uri) if not $schema_info;
-    die sprintf('bad $ref to %s: not a "%s"', $schema_info->{canonical_uri}, $entity_type)
-      if $entity_type and $entity_type ne 'schema'
+    die sprintf('bad $ref to %s: not a%s "%s"', $schema_info->{canonical_uri}, ($entity_type =~ /^[aeiou]/ ? 'n' : ''), $entity_type)
+      if $entity_type
         and $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne $entity_type;
 
     $entity_type //= $schema_info->{document}->get_entity_at_location($schema_info->{document_path});
@@ -796,6 +792,16 @@ sub _convert_request ($request) {
     $req->parse($request->as_string);
     return $req;
   }
+  elsif ($request->isa('Plack::Request')) {
+    my $req = Mojo::Message::Request->new->parse($request->env);
+    my $body = $request->content;
+    $req->parse($body) if length $body;
+    # Plack is unable to distinguish between %2F and /, so the raw (undecoded) uri can be passed
+    # here. see PSGI::FAQ
+    $req->url(Mojo::URL->new($request->env->{REQUEST_URI})) if exists $request->env->{REQUEST_URI};
+    return $req;
+  }
+
   croak 'unknown type '.ref($request);
 }
 
@@ -809,6 +815,18 @@ sub _convert_response ($response) {
     # the network should always have it set.
     return $res;
   }
+  elsif ($response->isa('Plack::Response')) {
+    my $res = Mojo::Message::Response->new;
+    $res->code($response->status);
+    my @headers = $response->headers->psgi_flatten->@*;
+    while (my ($name, $value) = splice(@headers, 0, 2)) {
+      $res->headers->header($name, $value);
+    }
+    my $body = $response->body;
+    $res->body($body) if length $body;
+    return $res;
+  }
+
   croak 'unknown type '.ref($response);
 }
 
@@ -835,7 +853,7 @@ OpenAPI::Modern - Validate HTTP requests and responses against an OpenAPI v3.1 d
 
 =head1 VERSION
 
-version 0.051
+version 0.052
 
 =head1 SYNOPSIS
 
@@ -1025,7 +1043,7 @@ The L<JSON::Schema::Modern> object to use for all URI resolution and JSON Schema
     },
   );
 
-Validates an L<HTTP::Request> or L<Mojo::Message::Request>
+Validates an L<HTTP::Request>, L<Plack::Request> or L<Mojo::Message::Request>
 object against the corresponding OpenAPI v3.1 document, returning a
 L<JSON::Schema::Modern::Result> object.
 
@@ -1045,7 +1063,7 @@ to improve performance.
     },
   );
 
-Validates an L<HTTP::Response> or L<Mojo::Message::Response>
+Validates an L<HTTP::Response>, L<Plack::Response> or L<Mojo::Message::Response>
 object against the corresponding OpenAPI v3.1 document, returning a
 L<JSON::Schema::Modern::Result> object.
 
@@ -1115,14 +1133,19 @@ path prefixes.
 =head2 recursive_get
 
 Given a uri or uri-reference, get the definition at that location, following any C<$ref>s along the
-way. Returns the data in scalar context, or a tuple of the data and the canonical URI of the
+way. Include the expected definition type
+(one of C<schema>, C<response>, C<parameter>, C<example>, C<request-body>, C<header>,
+C<security-scheme>, C<link>, C<callbacks>, or C<path-item>)
+for validation of the entire reference chain.
+
+Returns the data in scalar context, or a tuple of the data and the canonical URI of the
 referenced location in list context.
 
 If the provided location is relative, the main openapi document is used for the base URI.
 If you have a local json pointer you want to resolve, you can turn it into a uri-reference by
 prepending C<#>.
 
-  my $schema = $openapi->recursive_get('#/components/parameters/Content-Encoding');
+  my $schema = $openapi->recursive_get('#/components/parameters/Content-Encoding', 'parameter');
 
   # starts with a JSON::Schema::Modern object (TODO)
   my $schema = $js->recursive_get('https:///openapi_doc.yaml#/components/schemas/my_object')
